/***************************************************************************
 * FILE.C - Basic file functions                                           *
 * -----------------------------                                           *
 *                                                                         *
 * VCDID - CD-I filesystem driver                                          *
 * (c) 1997 - 1999 Vahur Sinijrv                                          *
 *                                                                         *
 * You are free to use any part of this source code, provided              *
 * that credit is given to author in such cases.                           *
 ***************************************************************************/
 
#include <basedef.h>
#include <vmm.h>
#include <error.h>
#include "vcdid.h"

#pragma VxD_PAGEABLE_CODE_SEG
#pragma VxD_PAGEABLE_DATA_SEG

#define SCSI_READCD 0xbe

#define ERRCHECK( cond, e ) { if ( cond ) { error = e; goto err; } }
#define GETVOLUME( p )      ((LPCDIFS_VOLUME)(p->ir_rh))

DWORD FindDirEntry( PDIRFILE pDirFile,
                    unsigned short *pattern,
                    PCDI_DIRENT *pDirEnt )
{
    _QWORD         qw;
    LPCDI_DIRENT   cur;
    unsigned short str[ MAX_PATH + 1 ];

    for( ;; )
    {
        cur = DirFile_GetNextEntry( pDirFile );
        if ( !cur ) break;

        qw = IFSMgr_BCSToUNI( str, cur->name, cur->name_len, BCS_WANSI );
        str[ qw.ddLower >> 1 ] = 0;
        if ( IFSMgr_MetaMatch( pattern, str, UFLG_NT ) )
        {
            *pDirEnt = cur;
            return NO_ERROR;
        }
    }

    return ERROR_FILE_NOT_FOUND;
}

unsigned short CDIToDosTime( BYTE* pCDITime, dos_time* pDosTime )
{
    int year;

    pDosTime->dt_time = ( pCDITime[ 5 ] >> 1  ) |
                        ( pCDITime[ 4 ] << 5  ) |
                        ( pCDITime[ 3 ] << 11 );

    pDosTime->dt_date = ( pCDITime[ 2 ]      ) |
                        ( pCDITime[ 1 ] << 5 );

    year = pCDITime[ 0 ] - 80;

    /* Check year limits */
    if ( year < 0 ) year = 0;
    else
        if ( year > 119 ) year = 119;

    pDosTime->dt_date |= year << 9;

    /* 10 ms precision */
   return ( pCDITime[ 5 ] & 1 ) ? 100 : 0;
}

DWORD DirEntryToFileInfo( PCDI_DIRENT pDirEnt, PFILEINFO pFi )
{
    PCDI_DIRENT_EXT pExt;

    pFi->start  = SWAPD( pDirEnt->lbn  );
    pFi->length = SWAPD( pDirEnt->size );

    pFi->creation_10ms = CDIToDosTime( pDirEnt->date, &pFi->creation_time );

    pExt = GET_DIRENT_EXT( pDirEnt );

    pFi->attr = FILE_ATTRIBUTE_READONLY;

    if ( pExt->attributes & CDI_FILE_ATTR_DIRECTORY )
        pFi->attr |= FILE_ATTRIBUTE_DIRECTORY;

    if ( pDirEnt->flags & 1 )
        pFi->attr |= FILE_ATTRIBUTE_HIDDEN;

    /* Test test test */
    if ( pExt->file_no )
    {
        pFi->length = ( ( pFi->length / COOKED_SECT_SIZE ) * SECT_SIZE ) +
                      ( pFi->length % COOKED_SECT_SIZE );
    }

    return NO_ERROR;
}

DWORD GetFileInfo( LPCDIFS_VOLUME l, ParsedPath *pPath, LPFILEINFO pFi )
{
    PCDI_DIRENT  pDirEnt;
    DIRFILE      dirfile;
    DWORD        dwDirSect;
    DWORD        dwResult;
    PathElement  *pel;

    zeromem( &dirfile, sizeof( dirfile ) );

    dwDirSect = FindDirRec( l, pPath );
    if ( !dwDirSect )
        return ERROR_PATH_NOT_FOUND;

    dwResult = IO_GetCachePtr( l, &pDirEnt, dwDirSect, 0 );
    if ( dwResult )
        return dwResult;

    pel = IFSLastElement( pPath );
    if ( pel->pe_length )
    {
        dwResult = DirFile_InitFromCache( &dirfile, l, pDirEnt );
        if ( dwResult != NO_ERROR )
            return dwResult;

        dwResult = FindDirEntry( &dirfile, pel->pe_unichars, &pDirEnt );
        if ( dwResult != NO_ERROR )
            return dwResult;
    }

    dwResult = DirEntryToFileInfo( pDirEnt, pFi );
    DirFile_Done( &dirfile );
    return dwResult;
}

DWORD __cdecl CDIFS_FileAttributes( pioreq p )
{
    FILEINFO fi;
    DWORD    error;

    switch( p->ir_flags )
    {
    case SET_ATTRIBUTES:
    case SET_ATTRIB_MODIFY_DATETIME:
    case SET_ATTRIB_LAST_ACCESS_DATETIME:
    case SET_ATTRIB_CREATION_DATETIME:
            p->ir_error = ERROR_WRITE_PROTECT;
            break;

    case GET_ATTRIBUTES:
    case GET_ATTRIB_COMP_FILESIZE:
    case GET_ATTRIB_CREATION_DATETIME:
    case GET_ATTRIB_MODIFY_DATETIME:

            error = GetFileInfo( p->ir_rh, p->ir_ppath, &fi );
            if ( error ) {
                p->ir_error = error;
                break;
            }

            switch( p->ir_flags )
            {
            case GET_ATTRIB_COMP_FILESIZE:
                p->ir_size = fi.length;
                break;

            case GET_ATTRIB_CREATION_DATETIME:
            case GET_ATTRIB_MODIFY_DATETIME:
                p->ir_dostime = fi.creation_time;
                p->ir_options = fi.creation_10ms;
                break;

            default:
                p->ir_attr = fi.attr;
            }

            p->ir_error = NO_ERROR;
            break;

    case GET_ATTRIB_LAST_ACCESS_DATETIME:
            p->ir_dostime.dt_time = 0;
            p->ir_dostime.dt_date = 0;
            p->ir_options         = 0;
            p->ir_error           = NO_ERROR;
            break;

    default:
            p->ir_error = ERROR_NOT_SUPPORTED;
    }

    return p->ir_error;
}

DWORD __cdecl CDIFS_Close( pioreq p )
{
    LPFILEINFO pFi;

    pFi = (LPFILEINFO)p->ir_fh;

    if ( p->ir_options & FILE_CLOSE_FOR_LEVEL4_LOCK )
    {
        p->ir_pos = 0;
    }

    if ( p->ir_flags == CLOSE_FINAL )
    {
        /**  Check if we have anything open **/
        if ( !GETVOLUME( p )->pFileList )
        {
            p->ir_error = ERROR_INVALID_HANDLE;
            return ERROR_INVALID_HANDLE;
        }

        _HeapFree( pFi->buf, 0 );

        /** Unlink this file from list of open files **/
        if ( pFi->pNext )
            pFi->pNext->pPrev = pFi->pPrev;
        else
            GETVOLUME( p )->pFileList = pFi->pPrev;

        if ( pFi->pPrev )
            pFi->pPrev->pNext = pFi->pNext;

        _HeapFree( pFi, 0 );
        GETVOLUME( p )->nFiles--;
    }

    p->ir_error = NO_ERROR;
    return NO_ERROR;
}

DWORD __cdecl CDIFS_Seek( pioreq p )
{
    LPFILEINFO lpfi;
    DWORD      pos;

    lpfi = (LPFILEINFO)p->ir_fh;
    if ( p->ir_flags == FILE_BEGIN ) pos = p->ir_pos;
    else pos = lpfi->length + p->ir_pos;

    if ( pos > lpfi->length ) p->ir_error = ERROR_SEEK;
    else {
        lpfi->pos   = pos;
        p->ir_pos   = pos;
        p->ir_error = NO_ERROR;
    }
    return p->ir_error;
}

DWORD __cdecl CDIFS_ReadFile( pioreq p )
{
    LPFILEINFO lpfi;
    DWORD      error, startsect, startoffs, endsect, endoffs;
    BYTE       *buf, *dataptr;
    int        i;

    lpfi = (LPFILEINFO)p->ir_fh;

    if ( p->ir_pos > lpfi->length ) {
        p->ir_pos    = lpfi->pos;
        p->ir_length = 0;
        p->ir_error  = ERROR_SEEK;
        return p->ir_error;
    }

    if ( p->ir_length + p->ir_pos > lpfi->length )
        p->ir_length = lpfi->length - p->ir_pos;

    /*
     *  See if we have anything to read
     */
    if ( !p->ir_length ) {
        lpfi->pos   = p->ir_pos;
        p->ir_error = NO_ERROR;
        return p->ir_error;
    }

    startsect    = p->ir_pos / SECT_SIZE + lpfi->start;
    startoffs    = p->ir_pos % SECT_SIZE;
    endsect      = (p->ir_pos + p->ir_length) / SECT_SIZE + lpfi->start;
    endoffs      = (p->ir_pos + p->ir_length) % SECT_SIZE;
    dataptr      = p->ir_data;
    p->ir_length = 0;
    lpfi->pos    = p->ir_pos;

    /*
     *  First part
     */
    if ( lpfi->bufsect != startsect ) {
        error = IO_ReadCached( p->ir_rh, lpfi->buf, startsect, IORF_CHAR_COMMAND );
        if ( error ) {
            p->ir_error  = error;
            return error;
        }
        lpfi->bufsect = startsect;
    }

    memcpy( dataptr,
            lpfi->buf + startoffs + SYNC_SIZE,
            (startsect == endsect) ? endoffs - startoffs :
                                     SECT_SIZE - startoffs
    );

    if ( startsect == endsect )
    {
        p->ir_length = endoffs - startoffs;
        lpfi->pos   += p->ir_length;
        p->ir_pos    = lpfi->pos;
        p->ir_error  = NO_ERROR;
        return NO_ERROR;
    }

    /*
     *  Middle part
     */
    p->ir_length = SECT_SIZE - startoffs;
    lpfi->pos   += p->ir_length;
    dataptr     += p->ir_length;

    for( i = 0; i < (endsect - startsect) - 1; i++ )
    {
        error = IO_ReadCached( p->ir_rh, lpfi->buf, lpfi->bufsect+1, IORF_CHAR_COMMAND );
        if ( error )
        {
            p->ir_pos   = lpfi->pos;
            p->ir_error = error;
            return error;
        }
        memcpy( dataptr, lpfi->buf + SYNC_SIZE, SECT_SIZE );
        p->ir_length += SECT_SIZE;
        lpfi->pos    += SECT_SIZE;
        dataptr      += SECT_SIZE;
        lpfi->bufsect++;
    }

    /*
     *  Last part
     */
    if ( endoffs )
    {
        error = IO_ReadCached( p->ir_rh, lpfi->buf, endsect, IORF_CHAR_COMMAND );
        if ( error )
        {
            p->ir_error = error;
            p->ir_pos   = lpfi->pos;
            return error;
        }
        memcpy( dataptr, lpfi->buf + SYNC_SIZE, endoffs );
        p->ir_length += endoffs;
        lpfi->pos    += endoffs;
        lpfi->bufsect = endsect;
    }

    p->ir_pos   = lpfi->pos;
    p->ir_error = NO_ERROR;
    return NO_ERROR;
}

DWORD __cdecl CDIFS_OpenFile( pioreq p )
{
    LPFILEINFO     lpfi;
    LPCDIFS_VOLUME pVol;
    DWORD          error;
    BYTE           *buf;

    lpfi = NULL;
    ERRCHECK( !p->ir_ppath, ERROR_NOT_SUPPORTED );  /* Fail special open for FAT filesystems */
    ERRCHECK( p->ir_options == ACTION_CREATEALWAYS, ERROR_ACCESS_DENIED )

    lpfi = _HeapAllocate( sizeof( FILEINFO ), HEAPSWAP | HEAPZEROINIT );
    ERRCHECK( !lpfi, ERROR_NOT_ENOUGH_MEMORY )

    error = GetFileInfo( p->ir_rh, p->ir_ppath, lpfi );

    switch( p->ir_options & ACTION_MASK )
    {
    case ACTION_CREATENEW:

        if ( error != ERROR_FILE_NOT_FOUND )
        {
            if ( error == NO_ERROR )
            {
                error = ERROR_FILE_EXISTS;
            }
        }
        else
        {
            error = ERROR_ACCESS_DENIED;
        }
        break;

    case ACTION_REPLACEEXISTING:
        if ( error == NO_ERROR )
        {
            error = ERROR_ACCESS_DENIED;
        }
        break;

    case ACTION_OPENALWAYS:
        if ( error == ERROR_FILE_NOT_FOUND )
        {
            error = ERROR_ACCESS_DENIED;
        }
        break;

    case ACTION_OPENEXISTING:
        break;

    default:
        error = ERROR_NOT_SUPPORTED;
    }

    if ( error ) goto err;

    ERRCHECK( p->ir_flags == ACCESS_WRITEONLY ||
              p->ir_flags == ACCESS_READWRITE,
              ERROR_ACCESS_DENIED );

    /*
     *  Allocate sector buffer for file
     */
    buf = _HeapAllocate( RAW_SECT_SIZE, 0 );
    ERRCHECK( !buf, ERROR_NOT_ENOUGH_MEMORY )

    lpfi->buf     = buf;
    lpfi->bufsect = 0;

    /*
     * Link this file to list of open files
     */
    pVol = (LPCDIFS_VOLUME)p->ir_rh;
    lpfi->pPrev = pVol->pFileList;
    lpfi->pNext = NULL;

    if ( lpfi->pPrev )
    {
        lpfi->pPrev->pNext = lpfi;
    }

    pVol->pFileList = lpfi;
    pVol->nFiles++;

    /*
     * File opened ok
     */
    p->ir_options         = ACTION_OPENED;
    p->ir_size            = lpfi->length;
    p->ir_attr            = FILE_ATTRIBUTE_READONLY;
    lpfi->pos             = 0;
    p->ir_fh              = lpfi;
    p->ir_dostime         = lpfi->creation_time;
    p->ir_hfunc->hf_read  = CDIFS_ReadFile;
    p->ir_hfunc->hf_write = CDIFS_FailWRP;
    p->ir_hfunc->hf_misc  = &cdifs_hndlopen;

    p->ir_error = NO_ERROR;
    return NO_ERROR;    

err:
    if ( lpfi ) _HeapFree( lpfi, 0 );
    p->ir_error = error;
    return error;
}

DWORD _cdecl CDIFS_EnumerateHandle( pioreq p )
{
    LPFILEINFO                 lpfi;
    LPCDIFS_VOLUME             l;
   _BY_HANDLE_FILE_INFORMATION *lpbhfi;

    lpfi = (LPFILEINFO)p->ir_fh;
    l    = (LPCDIFS_VOLUME)p->ir_rh;

    switch( p->ir_flags )
    {
    case ENUMH_GETFILEINFO:
        lpbhfi = (_BY_HANDLE_FILE_INFORMATION *)p->ir_data;
        lpbhfi->bhfi_dwFileAttributes = lpfi->attr;
        lpbhfi->bhfi_ftCreationTime   = IFSMgr_DosToWin32Time( lpfi->creation_time );
        lpbhfi->bhfi_ftLastWriteTime  = lpbhfi->bhfi_ftCreationTime;
        lpbhfi->bhfi_ftLastAccessTime.dwLowDateTime = 0;
        lpbhfi->bhfi_ftLastAccessTime.dwHighDateTime = 0;
        lpbhfi->bhfi_dwVolumeSerialNumber = l->serial;
        lpbhfi->bhfi_nFileSizeHigh  = 0;
        lpbhfi->bhfi_nFileSizeLow   = lpfi->length;
        lpbhfi->bhfi_nNumberOfLinks = 0;
        lpbhfi->bhfi_nFileIndexHigh = 0;
        lpbhfi->bhfi_nFileIndexLow  = 0;
        p->ir_error = NO_ERROR;
        break;
    default:
        p->ir_error = ERROR_NOT_SUPPORTED;
    }
    return p->ir_error;
}

DWORD _cdecl CDIFS_FileLocks( pioreq p )
{
    return CDIFS_Fail( p );
}

DWORD _cdecl CDIFS_FileTimes( pioreq p )
{
    LPFILEINFO lpfi = (LPFILEINFO)p->ir_fh;;

    switch( p->ir_flags )
    {
    case SET_MODIFY_DATETIME:
    case SET_LAST_ACCESS_DATETIME:
    case SET_CREATION_DATETIME:
        p->ir_error = ERROR_WRITE_PROTECT;
        break;

    case GET_CREATION_DATETIME:
        p->ir_options         = lpfi->creation_10ms;

    case GET_MODIFY_DATETIME:
        p->ir_dostime         = lpfi->creation_time;
        p->ir_error           = NO_ERROR;
        break;

    case GET_LAST_ACCESS_DATETIME:
        p->ir_options         = 0;
        p->ir_dostime.dt_time = 0;
        p->ir_dostime.dt_date = 0;
        p->ir_error           = NO_ERROR;
        break;

    default:
        p->ir_error = ERROR_NOT_SUPPORTED;
    }
    return p->ir_error;
}

DWORD _cdecl CDIFS_HandleInfo( pioreq p )
{
    return CDIFS_Fail( p );
}

DWORD _cdecl CDIFS_DirectDiskIO( pioreq p )
{
    return CDIFS_Fail( p );
}

