;Ϳ
;  IO.ASM - Input-output routines                               
;                                                               
;  VCDID - CD-I filesystem driver                               
;  (c) 1997 - 1999 Vahur Sinijrv                               
;                                                               
;  You are free to use any part of this source code, provided   
;  that credit is given to author in such cases.                
;
.386p
        .XLIST
        INCLUDE vmm.inc
        MASM=1
        INCLUDE blockdev.inc
        INCLUDE ios.inc
        INCLUDE ior.inc
        INCLUDE ilb.inc
        INCLUDE vcache.inc
        INCLUDE dcb.inc

        INCLUDE async.inc
        INCLUDE vcdid.inc

;
; Module definitions
;
LOCK_STRUCT      STRUC
  lock_addr      dd      ?  ; page # of 1st logical page locked
  lock_plcnt     dd      ?  ; count of pages locked
  lock_flags     dd      ?  ; flags
LOCK_STRUCT      ENDS

LOCKFLG_MPLOCK   EQU     0001H
LOCKFLG_DBLBFR   EQU     0002H

NUM_READ_RETRIES EQU     3

;
; Externals
;
extern _AllocIOP: near, _FreeIOP: near, _memcpy: near, _g_CacheID: byte

;
; External protos
;
AllocIOP PROTO C :DWORD
FreeIOP  PROTO C :DWORD
memcpy   PROTO C :DWORD, :DWORD, :DWORD

VxD_CODE_SEG

;----------------------------------
; Buffer discard procedure
;
;----------------------------------
IO_Discard      PROC
                PUBLIC IO_Discard

                ret
IO_Discard      ENDP

;-------------------------------------------------------
; Satisfy_DevCrit - Satisfy device criteria
;
; EBX = ptr to IOR
; ESI = ptr to LOCK_STRUCT
;
; MODIFY: EAX, ECX, EDX, EDI
;-------------------------------------------------------
Satisfy_DevCrit PROC
                test    [ esi ].lock_flags, LOCKFLG_DBLBFR
                jnz     @@dblbuffer

                push    ebx
                push    esi

                push    ebx
                call    [ cdifs_ilb ].ILB_io_criteria_rtn
                add     esp, 4
                or      eax, eax
                pop     esi
                pop     ebx
                jz      @@sdc_end
@@dblbuffer:
                or      [ ebx ].IOR_flags, IORF_DOUBLE_BUFFER
@@sdc_end:
                ret
Satisfy_DevCrit ENDP

;-------------------------------------------------------
; Lock_memory
;
; EAX = byte count - 0 is not valid value
; EBX = ptr to IOR
; ESI = ptr to LOCK_STRUCT
;
; MODIFY: EAX, ECX, EDX, EDI
;-------------------------------------------------------
Lock_Memory     PROC
                push    ebp

                push    ebx
                push    esi

                dec     eax
                mov     edx, [ ebx ].IOR_buffer_ptr   ; Get ptr to buffer
                add     eax, edx                      ; Last address to lock
                shr     edx, 12                       ; Get start page #
                shr     eax, 12                       ; Get last page #
                sub     eax, edx                      ; Get page count
                inc     eax                           ; Page count++
                mov     [ esi ].lock_plcnt, eax       ; Save pg count

                VMMcall _LinPageLock, < edx, eax, PAGEMARKDIRTY or PAGEMAPGLOBAL >
                pop     esi
                pop     ebx

                or      eax, eax                      ; Can't lock ?
                jz      @@failed                      ; Y: Double buffer

                mov     [ esi ].lock_addr, eax
                mov     edx, [ ebx ].IOR_buffer_ptr   ; Get buffer ptr
                and     edx, 0FFFH                    ; Get offset into page
                or      eax, edx                      ; Calc new lin address
                mov     [ ebx ].IOR_buffer_ptr, eax   ; Set new buffer ptr
                mov     [ esi ].lock_flags, LOCKFLG_MPLOCK
                jmp     @@return
@@failed:
                mov     [ esi ].lock_flags, LOCKFLG_DBLBFR
@@return:
                pop     ebp
                ret
Lock_Memory     ENDP

;------------------------------------------------------------
; Unlock_Memory
;
; ESI = ptr to LOCK_STRUCT
;
; MODIFY: ECX, EDX
;------------------------------------------------------------
Unlock_Memory   PROC
                test    [ esi ].lock_flags, LOCKFLG_MPLOCK
                jz      @@notlocked

                push    eax
                mov     eax, [ esi ].lock_addr
                mov     edx, [ esi ].lock_plcnt
                VMMcall _LinPageUnlock, < eax, edx, PAGEMAPGLOBAL >
                pop     eax
@@notlocked:
                ret
Unlock_Memory   ENDP

;----------------------------------------------------------------------------------
; IO_Read( LPCDIFS_VOLUME pVol,
;          void*          buf,
;          DWORD          block_no,
;          DWORD          count,
;          DWORD          flags
;----------------------------------------------------------------------------------
IO_Read         PROC C  pVol     :DWORD,
                        pBuf     :DWORD,
                        block_no :DWORD,
                        count    :DWORD,
                        flags    :DWORD
                PUBLIC  IO_Read

                LOCAL   LockData   :LOCK_STRUCT,
                        RetryCount :DWORD

                push    esi
                push    edi
                push    ebx
                push    ecx
                push    edx

                mov     [ RetryCount ], NUM_READ_RETRIES + 1
                invoke  AllocIOP, pVol

                mov     ebx, eax
                mov     [ ebx ].IOR_func, IOR_READ
                mov     eax, pBuf
                mov     edx, block_no
                mov     [ ebx ].IOR_buffer_ptr, eax
                mov     [ ebx ].IOR_start_addr, edx

                mov     eax, count
                mov     [ ebx ].IOR_flags, IORF_HIGH_PRIORITY or \
                                           IORF_VERSION_002 or \
                                           IORF_SYNC_COMMAND or \
                                           IORF_DATA_IN
                mov     ecx, flags
                or      [ ebx ].IOR_flags, ecx
                test    ecx, IORF_CHAR_COMMAND
                jnz     @@raw_read

                mov     [ ebx ].IOR_xfer_count, eax

                shl     eax, 11            ; Sector size is 2048
                jmp     @@lock_buffer
@@raw_read:
                mov     ecx, RAW_SECT_SIZE
                xor     edx, edx
                mul     ecx
                mov     [ ebx ].IOR_xfer_count, eax
@@lock_buffer:
                lea     esi, LockData
                call    Lock_Memory
                call    Satisfy_DevCrit
iord_retry:
                mov     esi, ebx
                mov     edi, pVol
                mov     edi, [ edi ].vi_dcb
                add     edi, DCB_bdd
                VxDCall IOS_SendCommand

                xor     eax, eax
                movzx   ecx, [ ebx ].IOR_status         ; Errors ?
                cmp     ecx, IORS_ERROR_DESIGNTR
                jb      @@end

                dec     [ RetryCount ]
                jnz     iord_retry

                mov     eax, ecx
                movzx   ecx, [ ebx ].IOR_func
                VxDCall IOSMapIORSToI24
@@end:
                lea     esi, LockData
                call    Unlock_Memory

                invoke  FreeIOP, ebx
                pop     edx
                pop     ecx
                pop     ebx
                pop     edi
                pop     esi
                ret
IO_Read         ENDP

;------------------------------------------------------------------------------
; IO_ReadCached( LPCDIFS_VOLUME pVol,
;                void*          buf,
;                DWORD          block_no,
;                DWORD          flags
;------------------------------------------------------------------------------
IO_ReadCached   PROC C  pVol     :DWORD,
                        pBuf     :DWORD,
                        block_no :DWORD,
                        flags    :DWORD

                PUBLIC  IO_ReadCached

                push    esi
                push    edi
                push    ebx
                push    ecx
                push    edx

                mov     al, VCFB_Create or VCFB_MakeMRU
                mov     ah, _g_CacheID
                mov     ebx, block_no
                mov     edi, pVol
                mov     edi, [ edi ].vi_serial
                VxDCall VCache_FindBlock
                mov     ebx, eax
                jnc     iorc_copy

                push    ebx
                invoke  IO_Read, pVol, eax, block_no, 1, flags
                pop     ebx
                or      eax, eax                        ; Read succeeded ?
                jz      iorc_copy                       ; Y: Copy cached data
                VxDCall VCache_FreeBlock                ; N: Free cache block
                jmp     iorc_return
iorc_copy:
                test    flags, IORF_CHAR_COMMAND        ; Raw sectors ?
                jz      iorc_cooked                     ; N:
                mov     eax, RAW_SECT_SIZE              ; Y:
                jmp     iorc_copy01
iorc_cooked:
                mov     eax, COOKED_SECT_SIZE
iorc_copy01:
                invoke  memcpy, pBuf, ebx, eax
                xor     eax, eax
iorc_return:
                pop     edx
                pop     ecx
                pop     ebx
                pop     edi
                pop     esi
                ret
IO_ReadCached   ENDP

;------------------------------------------------------------------------------
; IO_GetCachePtr( LPCDIFS_VOLUME pVol,
;                 void**         pPtr,
;                 DWORD          block_no,
;                 DWORD          flags
;------------------------------------------------------------------------------
IO_GetCachePtr  PROC C  pVol     :DWORD,
                        pPtr     :DWORD,
                        block_no :DWORD,
                        flags    :DWORD

                PUBLIC  IO_GetCachePtr

                push    esi
                push    edi
                push    ebx
                push    ecx
                push    edx

                mov     al, VCFB_Create or VCFB_MakeMRU
                mov     ah, _g_CacheID
                mov     ebx, block_no
                mov     edi, pVol
                mov     edi, [ edi ].vi_serial
                VxDCall VCache_FindBlock
                mov     ebx, eax
                jnc     iogcp_copy

                push    ebx
                invoke  IO_Read, pVol, eax, block_no, 1, flags
                pop     ebx
                or      eax, eax                        ; Read succeeded ?
                jz      iogcp_copy                      ; Y: Return cached data
                VxDCall VCache_FreeBlock                ; N: Free cache block
                jmp     iogcp_return
iogcp_copy:
                mov     eax, pPtr
                mov     [ eax ], ebx
                xor     eax, eax
iogcp_return:
                pop     edx
                pop     ecx
                pop     ebx
                pop     edi
                pop     esi
                ret
IO_GetCachePtr  ENDP

relpages        PROC
                cmp     dword ptr [esi].B_FSDData, ecx
                jnz     relpg_10
                push    ecx
                push    edx
                VxDCall VCache_FreeBlock
                pop     edx
                pop     ecx
relpg_10:
                ret
relpages        ENDP

;------------------------------------------------------------------------------
; IO_ReadCachedBytes( LPCDIFS_VOLUME pVol,
;                     void**         pPtr,
;                     DWORD          block_no,
;                     DWORD          bytes
;------------------------------------------------------------------------------
IO_ReadCachedBytes      PROC C  pVol     :DWORD,
                                pBuf     :DWORD,
                                block_no :DWORD,
                                bytes    :DWORD

                PUBLIC  IO_ReadCachedBytes
                LOCAL   pPtr

                mov     eax, bytes
                or      eax, eax
                jz      iorcb_end
iorcb_nextblock:
                invoke  IO_GetCachePtr, pVol, ADDR pPtr, block_no, 0
                test    eax, eax
                jnz     iorcb_end

                mov     edx, bytes
                dec     edx
                and     edx, COOKED_SECT_SIZE - 1
                inc     edx
                invoke  memcpy, pBuf, pPtr, edx
                inc     block_no
                sub     edx, COOKED_SECT_SIZE
                mov     bytes, edx
                ja      iorcb_nextblock
                xor     eax, eax
iorcb_end:
                ret
IO_ReadCachedBytes      ENDP

_IO_ReleaseCachePages   PROC
                PUBLIC  _IO_ReleaseCachePages

                mov     ecx, [esp+4]                    ; Get ptr to volume
                push    edi
                push    esi
                push    ebp
                push    ebx

                mov     ah, _g_CacheID
                mov     edx, OFFSET32 relpages
                VxDCall VCache_Enum

                pop     ebx
                pop     ebp
                pop     esi
                pop     edi
                ret
_IO_ReleaseCachePages   ENDP

VxD_CODE_ENDS

END
