/*
 * Borrowed from Linux headers
 */

#ifndef CDI_FS_H
#define CDI_FS_H

/* Number format */
#define INTEL

#ifdef INTEL
#define SWAPD(i) swapd(i)
#define SWAPW(i) swapw(i)
#define CDI_INT8B( ptr )    *(unsigned int *)(&ptr[4])
#define CDI_WORD( hi, lo )  ( (lo) << 8 | (hi) )
#else
#define SWAPD(i) i
#define SWAPW(i) i
#define CDI_INT8B( ptr )    *(unsigned int *)(&ptr[0])
#define CDI_WORD( hi, lo )  ( (hi) << 8 | (lo) )
#endif

#define CDIDCL(from, to) (to - from + 1)

struct cdi_volume_descriptor {
    char type[CDIDCL(1,1)];      /* 711 */
    char id[CDIDCL(2,6)];
    char version[CDIDCL(7,7)];
    char data[CDIDCL(8,2048)];
};

/* volume descriptor types */
#define CDI_VD_PRIMARY 1
#define CDI_VD_END     255

#define CDI_STANDARD_ID "CD-I "
#define CDI_SYSTEM_ID   "CD-RTOS"

#pragma pack( push, 1 )

struct cdi_primary_descriptor {
    char type                   [CDIDCL (  1,   1)]; /* 711 */
    char id                     [CDIDCL (  2,   6)];
    char version                [CDIDCL (  7,   7)]; /* 711 */
    char unused1                [CDIDCL (  8,   8)];
    char system_id              [CDIDCL (  9,  40)]; /* achars */
    char volume_id              [CDIDCL ( 41,  72)]; /* dchars */
    char unused2                [CDIDCL ( 73,  80)];
    char volume_space_size      [CDIDCL ( 81,  88)]; /* 733 */
    char unused3                [CDIDCL ( 89, 120)];
    char volume_set_size        [CDIDCL (121, 124)]; /* 723 */
    char volume_sequence_number [CDIDCL (125, 128)]; /* 723 */
    char logical_block_size     [CDIDCL (129, 132)]; /* 723 */
    char path_table_size        [CDIDCL (133, 140)]; /* 733 */
    char type_l_path_table      [CDIDCL (141, 144)]; /* 731 */
    char opt_type_l_path_table  [CDIDCL (145, 148)]; /* 731 */
    char type_m_path_table      [CDIDCL (149, 152)]; /* 732 */
    char opt_type_m_path_table  [CDIDCL (153, 156)]; /* 732 */
    char root_directory_record  [CDIDCL (157, 190)]; /* 9.1 */
    char album_id               [CDIDCL (191, 318)]; /* dchars */
    char publisher_id           [CDIDCL (319, 446)]; /* achars */
    char preparer_id            [CDIDCL (447, 574)]; /* achars */
    char application_id         [CDIDCL (575, 702)]; /* achars */
    char copyright_file_id      [CDIDCL (703, 739)]; /* 7.5 dchars */
    char abstract_file_id       [CDIDCL (740, 776)]; /* 7.5 dchars */
    char bibliographic_file_id  [CDIDCL (777, 813)]; /* 7.5 dchars */
    char creation_date          [CDIDCL (814, 830)]; /* 8.4.26.1 */
    char modification_date      [CDIDCL (831, 847)]; /* 8.4.26.1 */
    char expiration_date        [CDIDCL (848, 864)]; /* 8.4.26.1 */
    char effective_date         [CDIDCL (865, 881)]; /* 8.4.26.1 */
    char file_structure_version [CDIDCL (882, 882)]; /* 711 */
    char unused4                [CDIDCL (883, 883)];
    char application_data       [CDIDCL (884, 1395)];
    char unused5                [CDIDCL (1396, 2048)];
};

typedef struct cdi_primary_descriptor CDI_PVD, *LPCDI_PVD;

#define MAX_CDI_VOL_ID          CDIDCL (  41,  72 )
#define MAX_CDI_ALBUM_ID        CDIDCL ( 191, 318 )
#define MAX_CDI_PUBLISHER_ID    CDIDCL ( 319, 446 )
#define MAX_CDI_PREPARER_ID     CDIDCL ( 447, 574 )
#define MAX_CDI_APPLICATION_ID  CDIDCL ( 575, 702 )
#define MAX_CDI_DATETIME        CDIDCL ( 814, 830 )

struct  hs_volume_descriptor {
    char foo            [CDIDCL (  1,   8)]; /* 733 */
    char type           [CDIDCL (  9,   9)]; /* 711 */
    char id             [CDIDCL ( 10,  14)];
    char version        [CDIDCL ( 15,  15)]; /* 711 */
    char data           [CDIDCL ( 16,2048)];
};


struct hs_primary_descriptor {
    char foo                    [CDIDCL (  1,   8)]; /* 733 */
    char type                   [CDIDCL (  9,   9)]; /* 711 */
    char id                     [CDIDCL ( 10,  14)];
    char version                [CDIDCL ( 15,  15)]; /* 711 */
    char unused1                [CDIDCL ( 16,  16)]; /* 711 */
    char system_id              [CDIDCL ( 17,  48)]; /* achars */
    char volume_id              [CDIDCL ( 49,  80)]; /* dchars */
    char unused2                [CDIDCL ( 81,  88)]; /* 733 */
    char volume_space_size      [CDIDCL ( 89,  96)]; /* 733 */
    char unused3                [CDIDCL ( 97, 128)]; /* 733 */
    char volume_set_size        [CDIDCL (129, 132)]; /* 723 */
    char volume_sequence_number [CDIDCL (133, 136)]; /* 723 */
    char logical_block_size     [CDIDCL (137, 140)]; /* 723 */
    char path_table_size        [CDIDCL (141, 148)]; /* 733 */
    char type_l_path_table      [CDIDCL (149, 152)]; /* 731 */
    char unused4                [CDIDCL (153, 180)]; /* 733 */
    char root_directory_record  [CDIDCL (181, 214)]; /* 9.1 */
};

/* We use this to help us look up the parent inode numbers. */

struct cdi_path_table {
	unsigned char  name_len[2];	/* 721 */
	char extent[4];		/* 731 */
	char  parent[2];	/* 721 */
	char name[0];
};

typedef struct cdi_path_table CDI_PATH_TABLE, *LPCDI_PATH_TABLE;

/***************************************************************************
 *
 * Directory entry
 *
 **************************************************************************/
struct cdi_directory_record
{
    BYTE  length;
    BYTE  ext_attr_length;
    BYTE  reserved0[ 4 ];
    DWORD lbn;
    BYTE  reserved1[ 4 ];
    DWORD size;
    BYTE  date[ 6 ];
    BYTE  reserved2[ 1 ];
    BYTE  flags;
    WORD  interleave;
    BYTE  reserved3[ 2 ];
    WORD  album_set_seq_number;
    BYTE  name_len;
    BYTE  name[ 0 ];
};

struct cdi_directory_record_ext
{
    DWORD owner_id;
    WORD  attributes;
    BYTE  reserved0[ 2 ];
    BYTE  file_no;
    BYTE  reserved1[ 1 ];
};


/*
 * Macro to get to extension part of directory entry
 */
#define GET_DIRENT_EXT( p ) \
    ( (LPCDI_DIRENT_EXT)( (char*)(p) + (p)->length - sizeof( CDI_DIRENT_EXT ) ) )

#define PDIRENT_ADD( p, x ) ((LPCDI_DIRENT)(((char*)(p)) + (x)))
#define PDIRENT_NEXT( p )   ((LPCDI_DIRENT)(((char*)(p)) + (p)->length))

typedef struct cdi_directory_record
    CDI_DIRENT, *LPCDI_DIRENT, *PCDI_DIRENT;

typedef struct cdi_directory_record_ext
    CDI_DIRENT_EXT, *LPCDI_DIRENT_EXT, *PCDI_DIRENT_EXT;


/*
 * Directory entry attributes
 */
#define CDI_FILE_ATTR_OR         CDI_WORD( 0x00, 0x01 )
#define CDI_FILE_ATTR_OX         CDI_WORD( 0x00, 0x04 )
#define CDI_FILE_ATTR_GR         CDI_WORD( 0x00, 0x10 )
#define CDI_FILE_ATTR_GX         CDI_WORD( 0x00, 0x40 )
#define CDI_FILE_ATTR_WR         CDI_WORD( 0x01, 0x00 )
#define CDI_FILE_ATTR_WX         CDI_WORD( 0x04, 0x00 )
#define CDI_FILE_ATTR_CDDA       CDI_WORD( 0x40, 0x00 )
#define CDI_FILE_ATTR_DIRECTORY  CDI_WORD( 0x80, 0x00 )

/*
 * Directory entry flags
 */
#define CDI_FILE_FLAG_HIDDEN     0x01

/***************************************************************************
 *
 * Sector header
 *
 **************************************************************************/

typedef struct
{
    BYTE    file_no;
    BYTE    channel_no;
    BYTE    submode;
    BYTE    coding_info;
} CDI_SUBHEADER, *PCDI_SUBHEADER;

typedef struct
{
    BYTE          sync[ 12 ];
    BYTE          minutes;
    BYTE          seconds;
    BYTE          sectors;
    BYTE          mode;
    CDI_SUBHEADER subhdr[ 2 ];
} CDI_SECTORHEADER, *PCDI_SECTORHEADER;

#define SUBMODE_EOR     0x01
#define SUBMODE_V       0x02
#define SUBMODE_A       0x04
#define SUBMODE_D       0x08
#define SUBMODE_T       0x10
#define SUBMODE_F       0x20
#define SUBMODE_RT      0x40
#define SUBMODE_EOF     0x80

#define CDIFS_BLOCK_BITS 11
#define CDIFS_BLOCK_SIZE 2048

#pragma pack( pop )

#endif
