/***************************************************************************
 * CONFIG.C - Configuration releated functions                             *
 * -------------------------------------------                             *
 *                                                                         *
 * CDIFS - CD-I filesystem driver                                          *
 * (c) 1997 - 2000 Vahur Sinijrv                                          *
 *                                                                         *
 * You are free to use any part of this source code, provided              *
 * that credit is given to author in such cases.                           *
 ***************************************************************************/
 
#include <basedef.h>
#include <vmm.h>
#include <vmmreg.h>
#include "vcdid.h"
#include "cfg.h"

#pragma VxD_LOCKED_CODE_SEG
#pragma VxD_LOCKED_DATA_SEG

CDIFS_CONFIG g_config = { 1 };

char CONFIG_KEY_NAME[] = "Software\\VSi\\CDIFS";

//----------------------------------------------------------------------
// Name: config_read
// Desc: Read configuration record from registry.
//----------------------------------------------------------------------
void config_read( void )
{
    VMMHKEY hKey;
    LONG    result;
    DWORD   dwSize;

    result = VMM_RegOpenKey( HKEY_LOCAL_MACHINE,
        CONFIG_KEY_NAME,
        &hKey );

    if ( result != ERROR_SUCCESS )
        return;

    dwSize = sizeof( g_config );
    result = VMM_RegQueryValueEx( hKey,
        "Config",
        0,
        0,
        (PBYTE)&g_config,
        &dwSize );

    VMM_RegCloseKey( hKey );
}

//----------------------------------------------------------------------
// Name: config_write
// Desc: Write configuration record to registry. Returns ERROR_SUCCESS
//       if successful or error code otherwise.
//----------------------------------------------------------------------
LONG config_write( CDIFS_CONFIG* pConfig )
{
    VMMHKEY hKey;
    LONG    result;

    result = VMM_RegCreateKey( HKEY_LOCAL_MACHINE,
        CONFIG_KEY_NAME,
        &hKey );

    if ( result != ERROR_SUCCESS )
        return result;

    result = VMM_RegSetValueEx( hKey,
        "Config",
        0,
        REG_BINARY,
        (PBYTE)pConfig,
        sizeof( *pConfig )
    );

    VMM_RegCloseKey( hKey );

    return result;
}

//----------------------------------------------------------------------
// Name: config_set
// Desc: Set configuration to pConfig. If this function succeeds,
//       it returns ERROR_SUCCESS, otherwise it returns error code
//       and configuration is left unchanged.
//----------------------------------------------------------------------
LONG config_set( CDIFS_CONFIG* pConfig )
{
    LONG result;

    result = config_write( pConfig );
    if ( result == ERROR_SUCCESS )
    {
        memcpy( &g_config, pConfig, sizeof( CDIFS_CONFIG ) );
    }

    return result;
}
