/**********************************************************************
 * aboutpage.c
 *
 * Author:      Vahur Sinijrv
 * Description: About page of cpl property sheet
 *
 * (c) 1997 - 2000 Vahur Sinijrv
 *
 * You are free to use any part of this source code, provided
 * that credit is given to author in such cases.
 *
 *********************************************************************/

#include "headers.h"
#include "resource.h"
#include "error.h"
#include <richedit.h>

//
// Types
//
typedef struct
{
    LPBYTE lpData;
    LONG   lSize;
} RESDATA;

//
// Protos
//
static LRESULT CALLBACK dlgproc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam );

//
// Constants
//
static PROPSHEETPAGE _psp =
{
    sizeof( PROPSHEETPAGE ),
    PSP_DEFAULT,
    NULL,
    MAKEINTRESOURCE( IDD_ABOUT ),
    NULL,
    NULL,
    dlgproc,
    0,
    NULL,
    NULL
};

static TCHAR MODULE[] = "ABOUTPAGE";

//
// Module variables
//
static HWND     _hWnd;
static HWND     _hwndLogo;
static HPALETTE _hPalette;
static HANDLE   _hImage;
static int      _imageWidth;
static int      _imageHeight;

// --------------------------------------------------------------------
// Name: rtfCB()
// Desc: Callback to fill RTF control
// --------------------------------------------------------------------
DWORD CALLBACK rtfCB( DWORD dwCookie, LPBYTE pbBuff, LONG cb, LONG *pcb )
{
    RESDATA* p;
    
    p = (RESDATA*)dwCookie;

    if ( !p->lSize )
    {
        *pcb = 0;
        return 0;
    }

    if ( cb > p->lSize ) cb = p->lSize;
    memcpy( pbBuff, p->lpData, cb );
    p->lSize -= cb;
    p->lpData += cb;
    *pcb = cb;

    return 0;
}

// --------------------------------------------------------------------
// Name: createPalette()
// Desc: Create logical palette from DIB
// --------------------------------------------------------------------
static BOOL createPalette( HANDLE hImage )
{
    RGBQUAD palette[ 257 ];
    UINT    i;
    BYTE    tmp;

    HDC     hDC     = GetDC( _hWnd );
    HDC     hMemDC  = CreateCompatibleDC( hDC );
    HBITMAP hOldBmp = (HBITMAP) SelectObject( hMemDC, hImage );
    UINT    nColors = GetDIBColorTable( hMemDC, 0, 256, &palette[1] );

    SelectObject( hMemDC, hOldBmp );
    DeleteDC( hMemDC );
    ReleaseDC( _hWnd, hDC );

    ((LOGPALETTE*)&palette)->palVersion    = 0x300;
    ((LOGPALETTE*)&palette)->palNumEntries = nColors;

    /* Swap red & blue */
    for( i = 1; i <= nColors; i++ )
    {
        tmp = palette[i].rgbBlue;
        palette[i].rgbBlue = palette[i].rgbRed;
        palette[i].rgbRed  = tmp;
    }

    _hPalette = CreatePalette( (LOGPALETTE*)&palette );
    return _hPalette != NULL;
}

// --------------------------------------------------------------------
// Name: wm_initdialog
// Desc: Initialize about page
// --------------------------------------------------------------------
static BOOL wm_initdialog()
{
    HRSRC      hRTF;
    EDITSTREAM stream;
    RESDATA    resData;

    _hwndLogo = GetDlgItem( _hWnd, IDC_LOGO );

    /* Load logo image */
    _hImage = LoadImage( g_hInst,
        MAKEINTRESOURCE( IDB_ABOUT ),
        IMAGE_BITMAP,
        0,
        0,
        LR_CREATEDIBSECTION |
        LR_SHARED );

    if ( !_hImage )
    {
        ERR( "Could not load logo image" );
        return FALSE;
    }

    /* Set image control to this loaded picture */
    SendMessage( _hwndLogo, STM_SETIMAGE, IMAGE_BITMAP, (LPARAM)_hImage );

    /* Create logical palette from image */
    if ( !createPalette( _hImage ) )
    {
        ERR( "Could not create palette" );
        return FALSE;
    }

    /* Initialize Rich Edit control */
    hRTF = FindResource( g_hInst, MAKEINTRESOURCE( IDR_ABOUT_TEXT ), TEXT("RTF") );

    if ( hRTF )
    {
        resData.lSize  = SizeofResource( g_hInst, hRTF );
        resData.lpData = (LPBYTE)LockResource( LoadResource( g_hInst, hRTF ) );

        stream.dwCookie    = (DWORD)&resData;
        stream.pfnCallback = rtfCB;
        stream.dwError     = 0;
        SendDlgItemMessage( _hWnd, IDC_RICHEDIT, EM_STREAMIN, SF_RTF, (LPARAM)&stream );
        SendDlgItemMessage( _hWnd, IDC_RICHEDIT, EM_HIDESELECTION, 1, 0 );
    }

    return TRUE;
}

// --------------------------------------------------------------------
// Name: wm_initdialog
// Desc: Process WM_CTLCOLORSTATIC message. Called when logo bmp needs
//       updating. The palette stuff here is probably not polite.
// --------------------------------------------------------------------
static BOOL wm_ctlcolorstatic( HDC hDC, HWND hWnd )
{
    SelectPalette( hDC, _hPalette, TRUE );
    RealizePalette( hDC );
    return FALSE;
}

// --------------------------------------------------------------------
// Name: wm_destroy
// Desc: Just clean up when our page is destroyed
// --------------------------------------------------------------------
static BOOL wm_destroy()
{
    DeleteObject( _hPalette );
    DeleteObject( _hImage );
    return TRUE;
}

// --------------------------------------------------------------------
// Name: dlgproc
// Desc: Window procedure for about page
// --------------------------------------------------------------------
static LRESULT CALLBACK dlgproc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
    switch( uMsg )
    {
    case WM_INITDIALOG:
        _hWnd = hWnd;
        return wm_initdialog();

    case WM_CTLCOLORSTATIC:
        return wm_ctlcolorstatic( (HDC)wParam, (HWND)lParam );

    case WM_DESTROY:
        return wm_destroy();
    }

    return FALSE;
}

// --------------------------------------------------------------------
// Name: aboutpage_create
// Desc: Create handle to this page
// --------------------------------------------------------------------
HPROPSHEETPAGE aboutpage_create()
{
    _psp.hInstance = g_hInst;
    return CreatePropertySheetPage( &_psp );
}