/**********************************************************************
 * cdifsvxd.c
 *
 * Author:      Vahur Sinijrv
 * Description: Communication with CDIFS driver
 *
 * (c) 1997 - 2000 Vahur Sinijrv
 *
 * You are free to use any part of this source code, provided
 * that credit is given to author in such cases.
 *
 *********************************************************************/

#include "headers.h"
#include "cdifsvxd.h"

static HANDLE _handle = INVALID_HANDLE_VALUE;

// --------------------------------------------------------------------
// Name: ioctl
// Desc: Does cdifs ioctl 
// --------------------------------------------------------------------
__inline static BOOL ioctl( int cmd,
    LPVOID pInBuf,
    DWORD InBufSize,
    LPVOID pOutBuf,
    DWORD OutBufSize )
{
    DWORD dwRet;

    return DeviceIoControl( _handle,
        cmd,
        pInBuf,  InBufSize,
        pOutBuf, OutBufSize,
        &dwRet,
        NULL
    ) == TRUE;
}

// --------------------------------------------------------------------
// Name: cdifsvxd_init
// Desc: Initializes connection to driver
// --------------------------------------------------------------------
BOOL cdifsvxd_init()
{
    _handle = CreateFile( "\\\\.\\cdifs",
        GENERIC_READ,
        0,
        NULL,
        OPEN_EXISTING,
        0,
        NULL );

    return _handle != INVALID_HANDLE_VALUE;
}

// --------------------------------------------------------------------
// Name: cdifsvxd_done
// Desc: Breaks connection to driver
// --------------------------------------------------------------------
void cdifsvxd_done()
{
    if ( _handle != INVALID_HANDLE_VALUE )
    {
        CloseHandle( _handle );
    }
}

// --------------------------------------------------------------------
// Name: cdifsvxd_getMountedCount
// Desc: Gets number of currently mounted volumes
// --------------------------------------------------------------------
DWORD cdifsvxd_getMountedCount()
{
    DWORD dwCount = 0;

    ioctl( IOC_GET_MOUNT_COUNT, NULL, 0, &dwCount, sizeof( dwCount ) );
    return dwCount;
}

// --------------------------------------------------------------------
// Name: cdifsvxd_getMounted
// Desc: Gets information about currently mounted volumes
// --------------------------------------------------------------------
DWORD cdifsvxd_getMounted( PIOCS_CDIFS_VOLUMES volumes )
{
    ioctl( IOC_GET_MOUNTED_VOLUMES,
        &volumes->dwVolumes, sizeof( DWORD ),
        volumes,
        volumes->dwVolumes * sizeof( IOCS_CDIFS_VOLUME ) + sizeof( IOCS_CDIFS_VOLUMES ) );

    return volumes->dwVolumes;
}

// --------------------------------------------------------------------
// Name: cdifsvxd_getConfig
// Desc: Gets current CDIFS configuration
// --------------------------------------------------------------------
BOOL cdifsvxd_getConfig( CDIFS_CONFIG* pConfig )
{
    return ioctl( IOC_GET_CONFIG, NULL, 0, pConfig, sizeof( *pConfig ) );
}

// --------------------------------------------------------------------
// Name: cdifsvxd_setConfig
// Desc: Sets current CDIFS configuration
// --------------------------------------------------------------------
BOOL cdifsvxd_setConfig( CDIFS_CONFIG* pConfig )
{
    return ioctl( IOC_SET_CONFIG, pConfig, sizeof( *pConfig ), NULL, 0 );
}