/**********************************************************************
 * error.c
 *
 * Author:      Vahur Sinijrv
 * Description: Error handling and printouts
 *
 * (c) 1997 - 2000 Vahur Sinijrv
 *
 * You are free to use any part of this source code, provided
 * that credit is given to author in such cases.
 *
 *********************************************************************/

#include "headers.h"
#include <stdarg.h>

#define ERROR_MSG_LEN   256

// --------------------------------------------------------------------
// Name: format
// Desc: Format error message
// --------------------------------------------------------------------
static TCHAR* format( TCHAR* buf, const TCHAR* module, TCHAR* fmt, va_list ap )
{
    int    len;
    TCHAR* str;

    len = lstrlen( module ) * sizeof( TCHAR );
    memcpy( buf, module, len );
    str = buf + len;
    *str++ = ':';
    *str++ = ' ';

    str += wvsprintf( str, fmt, ap ) * sizeof( TCHAR );
    return str;
}

// --------------------------------------------------------------------
// Name: error
// Desc: Print error message
// --------------------------------------------------------------------
void __cdecl error( const TCHAR* module, TCHAR* fmt, ... )
{
    TCHAR   buf[ ERROR_MSG_LEN ];
    va_list ap;

    va_start( ap, fmt );

    format( buf, module, fmt, ap );

    MessageBox( NULL, buf, NULL, MB_OK | MB_ICONSTOP );
    va_end( ap );
}

// --------------------------------------------------------------------
// Name: errorw
// Desc: Print windows last error message
// --------------------------------------------------------------------
void __cdecl errorw( const TCHAR* module, TCHAR* fmt, ... )
{
    TCHAR   buf[ ERROR_MSG_LEN ];
    TCHAR*  str;
    DWORD   dwLastError;
    va_list ap;

    va_start( ap, fmt );
    dwLastError = GetLastError();
    str = format( buf, module, fmt, ap );
    *str++ = '\n';

    FormatMessage( FORMAT_MESSAGE_FROM_SYSTEM |
        FORMAT_MESSAGE_IGNORE_INSERTS,
        NULL,
        dwLastError,
        0,
        str,
        ERROR_MSG_LEN - ( str - buf ),
        NULL );

    MessageBox( NULL, buf, NULL, MB_OK | MB_ICONSTOP );
    va_end( ap );
}