/**********************************************************************
 * main.c
 *
 * Author:      Vahur Sinijrv
 * Description: Main module of CDIFS control panel
 *
 * (c) 1997 - 2000 Vahur Sinijrv
 *
 * You are free to use any part of this source code, provided
 * that credit is given to author in such cases.
 *
 *********************************************************************/

#include "headers.h"
#include "cdifsvxd.h"
#include "pages.h"
#include "resource.h"

HINSTANCE g_hInst;
HWND      g_hwnd = 0;

static TCHAR CAPTION[] = _T( "CDIFS Control Panel" );

// --------------------------------------------------------------------
// Name: propsheet
// Desc: Creates and runs property sheet
// --------------------------------------------------------------------
void propsheet( HWND hwndParent )
{
    PROPSHEETHEADER psh;
    HPROPSHEETPAGE  pages[ NUM_PAGES ];

    pages[ 0 ] = statuspage_create();
    pages[ 1 ] = settingspage_create();
    pages[ 2 ] = aboutpage_create();

    memset( &psh, 0, sizeof( psh ) );
    psh.dwSize     = sizeof( psh );
    psh.hInstance  = g_hInst;
    psh.hwndParent = hwndParent;
    psh.pszCaption = MAKEINTRESOURCE( IDS_NAME );
    psh.nPages     = NUM_PAGES;
    psh.phpage     = pages;

    PropertySheet( &psh );
}

// --------------------------------------------------------------------
// Name: DllMain
// Desc: Event handler for loading releted events
// --------------------------------------------------------------------
BOOL APIENTRY DllMain( HANDLE hModule,  DWORD  ul_reason_for_call, LPVOID lpReserved )
{
    switch (ul_reason_for_call)
    {
        case DLL_PROCESS_ATTACH:
            g_hInst = (HINSTANCE)hModule;
            break;

        case DLL_THREAD_ATTACH:
        case DLL_THREAD_DETACH:
        case DLL_PROCESS_DETACH:
            break;
    }
    return TRUE;
}

// --------------------------------------------------------------------
// Name: CplApplet
// Desc: Entry point for Control Panel
// --------------------------------------------------------------------
LONG APIENTRY CPlApplet( HWND hWnd, UINT uMsg, LONG lParam1, LONG lParam2 )
{
    LPCPLINFO cplinfo;

    switch( uMsg )
    {
    case CPL_INIT:
        LoadLibrary( _T("Riched32.dll") );
        return cdifsvxd_init();

    case CPL_GETCOUNT:
        return 1;

    case CPL_INQUIRE:
        {
            cplinfo = (LPCPLINFO)lParam2;
            cplinfo->idIcon = IDI_MAIN;
            cplinfo->idInfo = IDS_DESC;
            cplinfo->idName = IDS_NAME;
            cplinfo->lData  = 0;
            break;
        }

    case CPL_DBLCLK:
        propsheet( hWnd );
        break;
    }
    return 0;
}