/**********************************************************************
 * settingspage.c
 *
 * Author:      Vahur Sinijrv
 * Description: Settings page of CDIFS cpl
 *
 * (c) 1997 - 2000 Vahur Sinijrv
 *
 * You are free to use any part of this source code, provided
 * that credit is given to author in such cases.
 *
 *********************************************************************/

#include "headers.h"
#include "cdifsvxd.h"
#include "resource.h"

//
// Protos
//
static LRESULT CALLBACK dlgproc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam );

//
// Constants
//
static PROPSHEETPAGE _psp =
{
    sizeof( PROPSHEETPAGE ),
    PSP_DEFAULT,
    NULL,
    MAKEINTRESOURCE( IDD_SETTINGS ),
    NULL,
    NULL,
    dlgproc,
    0,
    NULL,
    NULL
};

//
// Module variables
//
static CDIFS_CONFIG _config;
static HWND         _hVolMountCB;
static HWND         _hParent;
static HWND         _hWnd;

// --------------------------------------------------------------------
// Name: wm_initdialog
// Desc: Initialize settings page
// --------------------------------------------------------------------
static BOOL wm_initdialog()
{
    _hParent     = GetParent( _hWnd );
    _hVolMountCB = GetDlgItem( _hWnd, IDC_ENABLE_VOL_MOUNT );

    cdifsvxd_getConfig( &_config );

    SendMessage( _hVolMountCB,
        BM_SETCHECK,
        _config.mount_flags ? BST_CHECKED : BST_UNCHECKED,
        0
    );

    return TRUE;
}

// --------------------------------------------------------------------
// Name: psn_apply
// Desc: Property sheet APPLY notification handler
// --------------------------------------------------------------------
static BOOL psn_apply()
{
    _config.mount_flags =
        SendMessage( _hVolMountCB, BM_GETCHECK, 0, 0 ) == BST_CHECKED ? 1 : 0;
    
    cdifsvxd_setConfig( &_config );
    SetWindowLong( _hWnd, DWL_MSGRESULT, PSNRET_NOERROR );
    return TRUE;
}

// --------------------------------------------------------------------
// Name: psn_killactive
// Desc: Property sheet KILLACTIVE notification handler
// --------------------------------------------------------------------
static BOOL psn_killactive()
{
    SetWindowLong( _hWnd, DWL_MSGRESULT, (LONG)FALSE );
    return TRUE;
}

// --------------------------------------------------------------------
// Name: dlgproc
// Desc: Window procedure for settings page
// --------------------------------------------------------------------
static LRESULT CALLBACK dlgproc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
    switch( uMsg )
    {
    case WM_INITDIALOG:
        _hWnd = hWnd;
        return wm_initdialog();

    case WM_COMMAND:
        switch( HIWORD( wParam ) )
        {
        case BN_CLICKED:
            SendMessage( _hParent, PSM_CHANGED, (WPARAM)_hWnd, 0 );
            return TRUE;
        }
        return FALSE;

    case WM_NOTIFY:
        switch (((NMHDR FAR *) lParam)->code)
        {
        case PSN_APPLY:
            return psn_apply();

        case PSN_KILLACTIVE:
            return psn_killactive();
        }
    }

    return FALSE;
}

// --------------------------------------------------------------------
// Name: settingspage_create
// Desc: Create handle to this page
// --------------------------------------------------------------------
HPROPSHEETPAGE settingspage_create()
{
    _psp.hInstance = g_hInst;
    return CreatePropertySheetPage( &_psp );
}